@@ -35,8 +35,16 @@ class YouTubeController(BaseController):
 
         Only works if there is no video playing.
         """
-        self.launch()
+        def callback():
+            """Plays requested video after app launched."""
+            self.start_play(youtube_id)
 
+        self.launch(callback)
+
+    def start_play(self, youtube_id):
+        """
+        Sends the play message to the YouTube app.
+        """
         msg = {
             "type": "flingVideo",
             "data": {
